/*
 * Generated by drgndoc.docstrings -H.
 *
 * Before Python 3.7, various docstring fields were defined as char * (see
 * https://bugs.python.org/issue28761). We still want the strings to be
 * read-only, so just cast away the const.
 */

extern const char drgn_DOC[];
#define drgn_DOC (char *)drgn_DOC
extern const char drgn_IntegerLike_DOC[];
#define drgn_IntegerLike_DOC (char *)drgn_IntegerLike_DOC
extern const char drgn_Path_DOC[];
#define drgn_Path_DOC (char *)drgn_Path_DOC
extern const char drgn_Program_DOC[];
#define drgn_Program_DOC (char *)drgn_Program_DOC
extern const char drgn_Program_flags_DOC[];
#define drgn_Program_flags_DOC (char *)drgn_Program_flags_DOC
extern const char drgn_Program_platform_DOC[];
#define drgn_Program_platform_DOC (char *)drgn_Program_platform_DOC
extern const char drgn_Program_language_DOC[];
#define drgn_Program_language_DOC (char *)drgn_Program_language_DOC
extern const char drgn_Program___getitem___DOC[];
#define drgn_Program___getitem___DOC (char *)drgn_Program___getitem___DOC
extern const char drgn_Program___contains___DOC[];
#define drgn_Program___contains___DOC (char *)drgn_Program___contains___DOC
extern const char drgn_Program_variable_DOC[];
#define drgn_Program_variable_DOC (char *)drgn_Program_variable_DOC
extern const char drgn_Program_constant_DOC[];
#define drgn_Program_constant_DOC (char *)drgn_Program_constant_DOC
extern const char drgn_Program_function_DOC[];
#define drgn_Program_function_DOC (char *)drgn_Program_function_DOC
extern const char drgn_Program_object_DOC[];
#define drgn_Program_object_DOC (char *)drgn_Program_object_DOC
extern const char drgn_Program_symbol_DOC[];
#define drgn_Program_symbol_DOC (char *)drgn_Program_symbol_DOC
extern const char drgn_Program_symbols_DOC[];
#define drgn_Program_symbols_DOC (char *)drgn_Program_symbols_DOC
extern const char drgn_Program_stack_trace_DOC[];
#define drgn_Program_stack_trace_DOC (char *)drgn_Program_stack_trace_DOC
extern const char drgn_Program_type_DOC[];
#define drgn_Program_type_DOC (char *)drgn_Program_type_DOC
extern const char drgn_Program_threads_DOC[];
#define drgn_Program_threads_DOC (char *)drgn_Program_threads_DOC
extern const char drgn_Program_thread_DOC[];
#define drgn_Program_thread_DOC (char *)drgn_Program_thread_DOC
extern const char drgn_Program_main_thread_DOC[];
#define drgn_Program_main_thread_DOC (char *)drgn_Program_main_thread_DOC
extern const char drgn_Program_crashed_thread_DOC[];
#define drgn_Program_crashed_thread_DOC (char *)drgn_Program_crashed_thread_DOC
extern const char drgn_Program_read_DOC[];
#define drgn_Program_read_DOC (char *)drgn_Program_read_DOC
extern const char drgn_Program_read_u8_DOC[];
#define drgn_Program_read_u8_DOC (char *)drgn_Program_read_u8_DOC
extern const char drgn_Program_read_u16_DOC[];
#define drgn_Program_read_u16_DOC (char *)drgn_Program_read_u16_DOC
extern const char drgn_Program_read_u32_DOC[];
#define drgn_Program_read_u32_DOC (char *)drgn_Program_read_u32_DOC
extern const char drgn_Program_read_u64_DOC[];
#define drgn_Program_read_u64_DOC (char *)drgn_Program_read_u64_DOC
extern const char drgn_Program_read_word_DOC[];
#define drgn_Program_read_word_DOC (char *)drgn_Program_read_word_DOC
extern const char drgn_Program_add_memory_segment_DOC[];
#define drgn_Program_add_memory_segment_DOC (char *)drgn_Program_add_memory_segment_DOC
extern const char drgn_Program_add_type_finder_DOC[];
#define drgn_Program_add_type_finder_DOC (char *)drgn_Program_add_type_finder_DOC
extern const char drgn_Program_add_object_finder_DOC[];
#define drgn_Program_add_object_finder_DOC (char *)drgn_Program_add_object_finder_DOC
extern const char drgn_Program_set_core_dump_DOC[];
#define drgn_Program_set_core_dump_DOC (char *)drgn_Program_set_core_dump_DOC
extern const char drgn_Program_set_kernel_DOC[];
#define drgn_Program_set_kernel_DOC (char *)drgn_Program_set_kernel_DOC
extern const char drgn_Program_set_pid_DOC[];
#define drgn_Program_set_pid_DOC (char *)drgn_Program_set_pid_DOC
extern const char drgn_Program_load_debug_info_DOC[];
#define drgn_Program_load_debug_info_DOC (char *)drgn_Program_load_debug_info_DOC
extern const char drgn_Program_load_default_debug_info_DOC[];
#define drgn_Program_load_default_debug_info_DOC (char *)drgn_Program_load_default_debug_info_DOC
extern const char drgn_Program_cache_DOC[];
#define drgn_Program_cache_DOC (char *)drgn_Program_cache_DOC
extern const char drgn_Program_void_type_DOC[];
#define drgn_Program_void_type_DOC (char *)drgn_Program_void_type_DOC
extern const char drgn_Program_int_type_DOC[];
#define drgn_Program_int_type_DOC (char *)drgn_Program_int_type_DOC
extern const char drgn_Program_bool_type_DOC[];
#define drgn_Program_bool_type_DOC (char *)drgn_Program_bool_type_DOC
extern const char drgn_Program_float_type_DOC[];
#define drgn_Program_float_type_DOC (char *)drgn_Program_float_type_DOC
extern const char drgn_Program_struct_type_DOC[];
#define drgn_Program_struct_type_DOC (char *)drgn_Program_struct_type_DOC
extern const char drgn_Program_union_type_DOC[];
#define drgn_Program_union_type_DOC (char *)drgn_Program_union_type_DOC
extern const char drgn_Program_class_type_DOC[];
#define drgn_Program_class_type_DOC (char *)drgn_Program_class_type_DOC
extern const char drgn_Program_enum_type_DOC[];
#define drgn_Program_enum_type_DOC (char *)drgn_Program_enum_type_DOC
extern const char drgn_Program_typedef_type_DOC[];
#define drgn_Program_typedef_type_DOC (char *)drgn_Program_typedef_type_DOC
extern const char drgn_Program_pointer_type_DOC[];
#define drgn_Program_pointer_type_DOC (char *)drgn_Program_pointer_type_DOC
extern const char drgn_Program_array_type_DOC[];
#define drgn_Program_array_type_DOC (char *)drgn_Program_array_type_DOC
extern const char drgn_Program_function_type_DOC[];
#define drgn_Program_function_type_DOC (char *)drgn_Program_function_type_DOC
extern const char drgn_ProgramFlags_DOC[];
#define drgn_ProgramFlags_DOC (char *)drgn_ProgramFlags_DOC
extern const char drgn_ProgramFlags_IS_LINUX_KERNEL_DOC[];
#define drgn_ProgramFlags_IS_LINUX_KERNEL_DOC (char *)drgn_ProgramFlags_IS_LINUX_KERNEL_DOC
extern const char drgn_ProgramFlags_IS_LIVE_DOC[];
#define drgn_ProgramFlags_IS_LIVE_DOC (char *)drgn_ProgramFlags_IS_LIVE_DOC
extern const char drgn_FindObjectFlags_DOC[];
#define drgn_FindObjectFlags_DOC (char *)drgn_FindObjectFlags_DOC
extern const char drgn_FindObjectFlags_CONSTANT_DOC[];
#define drgn_FindObjectFlags_CONSTANT_DOC (char *)drgn_FindObjectFlags_CONSTANT_DOC
extern const char drgn_FindObjectFlags_FUNCTION_DOC[];
#define drgn_FindObjectFlags_FUNCTION_DOC (char *)drgn_FindObjectFlags_FUNCTION_DOC
extern const char drgn_FindObjectFlags_VARIABLE_DOC[];
#define drgn_FindObjectFlags_VARIABLE_DOC (char *)drgn_FindObjectFlags_VARIABLE_DOC
extern const char drgn_FindObjectFlags_ANY_DOC[];
#define drgn_FindObjectFlags_ANY_DOC (char *)drgn_FindObjectFlags_ANY_DOC
extern const char drgn_Thread_DOC[];
#define drgn_Thread_DOC (char *)drgn_Thread_DOC
extern const char drgn_Thread_tid_DOC[];
#define drgn_Thread_tid_DOC (char *)drgn_Thread_tid_DOC
extern const char drgn_Thread_object_DOC[];
#define drgn_Thread_object_DOC (char *)drgn_Thread_object_DOC
extern const char drgn_Thread_stack_trace_DOC[];
#define drgn_Thread_stack_trace_DOC (char *)drgn_Thread_stack_trace_DOC
extern const char drgn_filename_matches_DOC[];
#define drgn_filename_matches_DOC (char *)drgn_filename_matches_DOC
extern const char drgn_program_from_core_dump_DOC[];
#define drgn_program_from_core_dump_DOC (char *)drgn_program_from_core_dump_DOC
extern const char drgn_program_from_kernel_DOC[];
#define drgn_program_from_kernel_DOC (char *)drgn_program_from_kernel_DOC
extern const char drgn_program_from_pid_DOC[];
#define drgn_program_from_pid_DOC (char *)drgn_program_from_pid_DOC
extern const char drgn_Platform_DOC[];
#define drgn_Platform_DOC (char *)drgn_Platform_DOC
extern const char drgn_Platform_arch_DOC[];
#define drgn_Platform_arch_DOC (char *)drgn_Platform_arch_DOC
extern const char drgn_Platform_flags_DOC[];
#define drgn_Platform_flags_DOC (char *)drgn_Platform_flags_DOC
extern const char drgn_Platform_registers_DOC[];
#define drgn_Platform_registers_DOC (char *)drgn_Platform_registers_DOC
extern const char drgn_Architecture_DOC[];
#define drgn_Architecture_DOC (char *)drgn_Architecture_DOC
extern const char drgn_Architecture_X86_64_DOC[];
#define drgn_Architecture_X86_64_DOC (char *)drgn_Architecture_X86_64_DOC
extern const char drgn_Architecture_I386_DOC[];
#define drgn_Architecture_I386_DOC (char *)drgn_Architecture_I386_DOC
extern const char drgn_Architecture_AARCH64_DOC[];
#define drgn_Architecture_AARCH64_DOC (char *)drgn_Architecture_AARCH64_DOC
extern const char drgn_Architecture_ARM_DOC[];
#define drgn_Architecture_ARM_DOC (char *)drgn_Architecture_ARM_DOC
extern const char drgn_Architecture_PPC64_DOC[];
#define drgn_Architecture_PPC64_DOC (char *)drgn_Architecture_PPC64_DOC
extern const char drgn_Architecture_RISCV64_DOC[];
#define drgn_Architecture_RISCV64_DOC (char *)drgn_Architecture_RISCV64_DOC
extern const char drgn_Architecture_RISCV32_DOC[];
#define drgn_Architecture_RISCV32_DOC (char *)drgn_Architecture_RISCV32_DOC
extern const char drgn_Architecture_UNKNOWN_DOC[];
#define drgn_Architecture_UNKNOWN_DOC (char *)drgn_Architecture_UNKNOWN_DOC
extern const char drgn_PlatformFlags_DOC[];
#define drgn_PlatformFlags_DOC (char *)drgn_PlatformFlags_DOC
extern const char drgn_PlatformFlags_IS_64_BIT_DOC[];
#define drgn_PlatformFlags_IS_64_BIT_DOC (char *)drgn_PlatformFlags_IS_64_BIT_DOC
extern const char drgn_PlatformFlags_IS_LITTLE_ENDIAN_DOC[];
#define drgn_PlatformFlags_IS_LITTLE_ENDIAN_DOC (char *)drgn_PlatformFlags_IS_LITTLE_ENDIAN_DOC
extern const char drgn_Register_DOC[];
#define drgn_Register_DOC (char *)drgn_Register_DOC
extern const char drgn_Register_names_DOC[];
#define drgn_Register_names_DOC (char *)drgn_Register_names_DOC
extern const char drgn_host_platform_DOC[];
#define drgn_host_platform_DOC (char *)drgn_host_platform_DOC
extern const char drgn_Language_DOC[];
#define drgn_Language_DOC (char *)drgn_Language_DOC
extern const char drgn_Language_name_DOC[];
#define drgn_Language_name_DOC (char *)drgn_Language_name_DOC
extern const char drgn_Language_C_DOC[];
#define drgn_Language_C_DOC (char *)drgn_Language_C_DOC
extern const char drgn_Language_CPP_DOC[];
#define drgn_Language_CPP_DOC (char *)drgn_Language_CPP_DOC
extern const char drgn_Object_DOC[];
#define drgn_Object_DOC (char *)drgn_Object_DOC
extern const char drgn_Object_prog__DOC[];
#define drgn_Object_prog__DOC (char *)drgn_Object_prog__DOC
extern const char drgn_Object_type__DOC[];
#define drgn_Object_type__DOC (char *)drgn_Object_type__DOC
extern const char drgn_Object_absent__DOC[];
#define drgn_Object_absent__DOC (char *)drgn_Object_absent__DOC
extern const char drgn_Object_address__DOC[];
#define drgn_Object_address__DOC (char *)drgn_Object_address__DOC
extern const char drgn_Object_bit_offset__DOC[];
#define drgn_Object_bit_offset__DOC (char *)drgn_Object_bit_offset__DOC
extern const char drgn_Object_bit_field_size__DOC[];
#define drgn_Object_bit_field_size__DOC (char *)drgn_Object_bit_field_size__DOC
extern const char drgn_Object___getattribute___DOC[];
#define drgn_Object___getattribute___DOC (char *)drgn_Object___getattribute___DOC
extern const char drgn_Object___getitem___DOC[];
#define drgn_Object___getitem___DOC (char *)drgn_Object___getitem___DOC
extern const char drgn_Object___len___DOC[];
#define drgn_Object___len___DOC (char *)drgn_Object___len___DOC
extern const char drgn_Object_value__DOC[];
#define drgn_Object_value__DOC (char *)drgn_Object_value__DOC
extern const char drgn_Object_string__DOC[];
#define drgn_Object_string__DOC (char *)drgn_Object_string__DOC
extern const char drgn_Object_member__DOC[];
#define drgn_Object_member__DOC (char *)drgn_Object_member__DOC
extern const char drgn_Object_address_of__DOC[];
#define drgn_Object_address_of__DOC (char *)drgn_Object_address_of__DOC
extern const char drgn_Object_read__DOC[];
#define drgn_Object_read__DOC (char *)drgn_Object_read__DOC
extern const char drgn_Object_to_bytes__DOC[];
#define drgn_Object_to_bytes__DOC (char *)drgn_Object_to_bytes__DOC
extern const char drgn_Object_from_bytes__DOC[];
#define drgn_Object_from_bytes__DOC (char *)drgn_Object_from_bytes__DOC
extern const char drgn_Object_format__DOC[];
#define drgn_Object_format__DOC (char *)drgn_Object_format__DOC
extern const char drgn_NULL_DOC[];
#define drgn_NULL_DOC (char *)drgn_NULL_DOC
extern const char drgn_cast_DOC[];
#define drgn_cast_DOC (char *)drgn_cast_DOC
extern const char drgn_reinterpret_DOC[];
#define drgn_reinterpret_DOC (char *)drgn_reinterpret_DOC
extern const char drgn_container_of_DOC[];
#define drgn_container_of_DOC (char *)drgn_container_of_DOC
extern const char drgn_Symbol_DOC[];
#define drgn_Symbol_DOC (char *)drgn_Symbol_DOC
extern const char drgn_Symbol_name_DOC[];
#define drgn_Symbol_name_DOC (char *)drgn_Symbol_name_DOC
extern const char drgn_Symbol_address_DOC[];
#define drgn_Symbol_address_DOC (char *)drgn_Symbol_address_DOC
extern const char drgn_Symbol_size_DOC[];
#define drgn_Symbol_size_DOC (char *)drgn_Symbol_size_DOC
extern const char drgn_Symbol_binding_DOC[];
#define drgn_Symbol_binding_DOC (char *)drgn_Symbol_binding_DOC
extern const char drgn_Symbol_kind_DOC[];
#define drgn_Symbol_kind_DOC (char *)drgn_Symbol_kind_DOC
extern const char drgn_SymbolBinding_DOC[];
#define drgn_SymbolBinding_DOC (char *)drgn_SymbolBinding_DOC
extern const char drgn_SymbolBinding_UNKNOWN_DOC[];
#define drgn_SymbolBinding_UNKNOWN_DOC (char *)drgn_SymbolBinding_UNKNOWN_DOC
extern const char drgn_SymbolBinding_LOCAL_DOC[];
#define drgn_SymbolBinding_LOCAL_DOC (char *)drgn_SymbolBinding_LOCAL_DOC
extern const char drgn_SymbolBinding_GLOBAL_DOC[];
#define drgn_SymbolBinding_GLOBAL_DOC (char *)drgn_SymbolBinding_GLOBAL_DOC
extern const char drgn_SymbolBinding_WEAK_DOC[];
#define drgn_SymbolBinding_WEAK_DOC (char *)drgn_SymbolBinding_WEAK_DOC
extern const char drgn_SymbolBinding_UNIQUE_DOC[];
#define drgn_SymbolBinding_UNIQUE_DOC (char *)drgn_SymbolBinding_UNIQUE_DOC
extern const char drgn_SymbolKind_DOC[];
#define drgn_SymbolKind_DOC (char *)drgn_SymbolKind_DOC
extern const char drgn_SymbolKind_UNKNOWN_DOC[];
#define drgn_SymbolKind_UNKNOWN_DOC (char *)drgn_SymbolKind_UNKNOWN_DOC
extern const char drgn_SymbolKind_OBJECT_DOC[];
#define drgn_SymbolKind_OBJECT_DOC (char *)drgn_SymbolKind_OBJECT_DOC
extern const char drgn_SymbolKind_FUNC_DOC[];
#define drgn_SymbolKind_FUNC_DOC (char *)drgn_SymbolKind_FUNC_DOC
extern const char drgn_SymbolKind_SECTION_DOC[];
#define drgn_SymbolKind_SECTION_DOC (char *)drgn_SymbolKind_SECTION_DOC
extern const char drgn_SymbolKind_FILE_DOC[];
#define drgn_SymbolKind_FILE_DOC (char *)drgn_SymbolKind_FILE_DOC
extern const char drgn_SymbolKind_COMMON_DOC[];
#define drgn_SymbolKind_COMMON_DOC (char *)drgn_SymbolKind_COMMON_DOC
extern const char drgn_SymbolKind_TLS_DOC[];
#define drgn_SymbolKind_TLS_DOC (char *)drgn_SymbolKind_TLS_DOC
extern const char drgn_SymbolKind_IFUNC_DOC[];
#define drgn_SymbolKind_IFUNC_DOC (char *)drgn_SymbolKind_IFUNC_DOC
extern const char drgn_StackTrace_DOC[];
#define drgn_StackTrace_DOC (char *)drgn_StackTrace_DOC
extern const char drgn_StackFrame_DOC[];
#define drgn_StackFrame_DOC (char *)drgn_StackFrame_DOC
extern const char drgn_StackFrame_name_DOC[];
#define drgn_StackFrame_name_DOC (char *)drgn_StackFrame_name_DOC
extern const char drgn_StackFrame_is_inline_DOC[];
#define drgn_StackFrame_is_inline_DOC (char *)drgn_StackFrame_is_inline_DOC
extern const char drgn_StackFrame_interrupted_DOC[];
#define drgn_StackFrame_interrupted_DOC (char *)drgn_StackFrame_interrupted_DOC
extern const char drgn_StackFrame_pc_DOC[];
#define drgn_StackFrame_pc_DOC (char *)drgn_StackFrame_pc_DOC
extern const char drgn_StackFrame___getitem___DOC[];
#define drgn_StackFrame___getitem___DOC (char *)drgn_StackFrame___getitem___DOC
extern const char drgn_StackFrame___contains___DOC[];
#define drgn_StackFrame___contains___DOC (char *)drgn_StackFrame___contains___DOC
extern const char drgn_StackFrame_source_DOC[];
#define drgn_StackFrame_source_DOC (char *)drgn_StackFrame_source_DOC
extern const char drgn_StackFrame_symbol_DOC[];
#define drgn_StackFrame_symbol_DOC (char *)drgn_StackFrame_symbol_DOC
extern const char drgn_StackFrame_register_DOC[];
#define drgn_StackFrame_register_DOC (char *)drgn_StackFrame_register_DOC
extern const char drgn_StackFrame_registers_DOC[];
#define drgn_StackFrame_registers_DOC (char *)drgn_StackFrame_registers_DOC
extern const char drgn_Type_DOC[];
#define drgn_Type_DOC (char *)drgn_Type_DOC
extern const char drgn_Type_prog_DOC[];
#define drgn_Type_prog_DOC (char *)drgn_Type_prog_DOC
extern const char drgn_Type_kind_DOC[];
#define drgn_Type_kind_DOC (char *)drgn_Type_kind_DOC
extern const char drgn_Type_primitive_DOC[];
#define drgn_Type_primitive_DOC (char *)drgn_Type_primitive_DOC
extern const char drgn_Type_qualifiers_DOC[];
#define drgn_Type_qualifiers_DOC (char *)drgn_Type_qualifiers_DOC
extern const char drgn_Type_language_DOC[];
#define drgn_Type_language_DOC (char *)drgn_Type_language_DOC
extern const char drgn_Type_name_DOC[];
#define drgn_Type_name_DOC (char *)drgn_Type_name_DOC
extern const char drgn_Type_tag_DOC[];
#define drgn_Type_tag_DOC (char *)drgn_Type_tag_DOC
extern const char drgn_Type_size_DOC[];
#define drgn_Type_size_DOC (char *)drgn_Type_size_DOC
extern const char drgn_Type_length_DOC[];
#define drgn_Type_length_DOC (char *)drgn_Type_length_DOC
extern const char drgn_Type_is_signed_DOC[];
#define drgn_Type_is_signed_DOC (char *)drgn_Type_is_signed_DOC
extern const char drgn_Type_byteorder_DOC[];
#define drgn_Type_byteorder_DOC (char *)drgn_Type_byteorder_DOC
extern const char drgn_Type_type_DOC[];
#define drgn_Type_type_DOC (char *)drgn_Type_type_DOC
extern const char drgn_Type_members_DOC[];
#define drgn_Type_members_DOC (char *)drgn_Type_members_DOC
extern const char drgn_Type_enumerators_DOC[];
#define drgn_Type_enumerators_DOC (char *)drgn_Type_enumerators_DOC
extern const char drgn_Type_parameters_DOC[];
#define drgn_Type_parameters_DOC (char *)drgn_Type_parameters_DOC
extern const char drgn_Type_is_variadic_DOC[];
#define drgn_Type_is_variadic_DOC (char *)drgn_Type_is_variadic_DOC
extern const char drgn_Type_template_parameters_DOC[];
#define drgn_Type_template_parameters_DOC (char *)drgn_Type_template_parameters_DOC
extern const char drgn_Type_type_name_DOC[];
#define drgn_Type_type_name_DOC (char *)drgn_Type_type_name_DOC
extern const char drgn_Type_is_complete_DOC[];
#define drgn_Type_is_complete_DOC (char *)drgn_Type_is_complete_DOC
extern const char drgn_Type_qualified_DOC[];
#define drgn_Type_qualified_DOC (char *)drgn_Type_qualified_DOC
extern const char drgn_Type_unqualified_DOC[];
#define drgn_Type_unqualified_DOC (char *)drgn_Type_unqualified_DOC
extern const char drgn_Type_member_DOC[];
#define drgn_Type_member_DOC (char *)drgn_Type_member_DOC
extern const char drgn_Type_has_member_DOC[];
#define drgn_Type_has_member_DOC (char *)drgn_Type_has_member_DOC
extern const char drgn_TypeMember_DOC[];
#define drgn_TypeMember_DOC (char *)drgn_TypeMember_DOC
extern const char drgn_TypeMember_object_DOC[];
#define drgn_TypeMember_object_DOC (char *)drgn_TypeMember_object_DOC
extern const char drgn_TypeMember_type_DOC[];
#define drgn_TypeMember_type_DOC (char *)drgn_TypeMember_type_DOC
extern const char drgn_TypeMember_name_DOC[];
#define drgn_TypeMember_name_DOC (char *)drgn_TypeMember_name_DOC
extern const char drgn_TypeMember_bit_offset_DOC[];
#define drgn_TypeMember_bit_offset_DOC (char *)drgn_TypeMember_bit_offset_DOC
extern const char drgn_TypeMember_offset_DOC[];
#define drgn_TypeMember_offset_DOC (char *)drgn_TypeMember_offset_DOC
extern const char drgn_TypeMember_bit_field_size_DOC[];
#define drgn_TypeMember_bit_field_size_DOC (char *)drgn_TypeMember_bit_field_size_DOC
extern const char drgn_TypeEnumerator_DOC[];
#define drgn_TypeEnumerator_DOC (char *)drgn_TypeEnumerator_DOC
extern const char drgn_TypeEnumerator_name_DOC[];
#define drgn_TypeEnumerator_name_DOC (char *)drgn_TypeEnumerator_name_DOC
extern const char drgn_TypeEnumerator_value_DOC[];
#define drgn_TypeEnumerator_value_DOC (char *)drgn_TypeEnumerator_value_DOC
extern const char drgn_TypeParameter_DOC[];
#define drgn_TypeParameter_DOC (char *)drgn_TypeParameter_DOC
extern const char drgn_TypeParameter_default_argument_DOC[];
#define drgn_TypeParameter_default_argument_DOC (char *)drgn_TypeParameter_default_argument_DOC
extern const char drgn_TypeParameter_type_DOC[];
#define drgn_TypeParameter_type_DOC (char *)drgn_TypeParameter_type_DOC
extern const char drgn_TypeParameter_name_DOC[];
#define drgn_TypeParameter_name_DOC (char *)drgn_TypeParameter_name_DOC
extern const char drgn_TypeTemplateParameter_DOC[];
#define drgn_TypeTemplateParameter_DOC (char *)drgn_TypeTemplateParameter_DOC
extern const char drgn_TypeTemplateParameter_argument_DOC[];
#define drgn_TypeTemplateParameter_argument_DOC (char *)drgn_TypeTemplateParameter_argument_DOC
extern const char drgn_TypeTemplateParameter_name_DOC[];
#define drgn_TypeTemplateParameter_name_DOC (char *)drgn_TypeTemplateParameter_name_DOC
extern const char drgn_TypeTemplateParameter_is_default_DOC[];
#define drgn_TypeTemplateParameter_is_default_DOC (char *)drgn_TypeTemplateParameter_is_default_DOC
extern const char drgn_TypeKind_DOC[];
#define drgn_TypeKind_DOC (char *)drgn_TypeKind_DOC
extern const char drgn_TypeKind_VOID_DOC[];
#define drgn_TypeKind_VOID_DOC (char *)drgn_TypeKind_VOID_DOC
extern const char drgn_TypeKind_INT_DOC[];
#define drgn_TypeKind_INT_DOC (char *)drgn_TypeKind_INT_DOC
extern const char drgn_TypeKind_BOOL_DOC[];
#define drgn_TypeKind_BOOL_DOC (char *)drgn_TypeKind_BOOL_DOC
extern const char drgn_TypeKind_FLOAT_DOC[];
#define drgn_TypeKind_FLOAT_DOC (char *)drgn_TypeKind_FLOAT_DOC
extern const char drgn_TypeKind_COMPLEX_DOC[];
#define drgn_TypeKind_COMPLEX_DOC (char *)drgn_TypeKind_COMPLEX_DOC
extern const char drgn_TypeKind_STRUCT_DOC[];
#define drgn_TypeKind_STRUCT_DOC (char *)drgn_TypeKind_STRUCT_DOC
extern const char drgn_TypeKind_UNION_DOC[];
#define drgn_TypeKind_UNION_DOC (char *)drgn_TypeKind_UNION_DOC
extern const char drgn_TypeKind_CLASS_DOC[];
#define drgn_TypeKind_CLASS_DOC (char *)drgn_TypeKind_CLASS_DOC
extern const char drgn_TypeKind_ENUM_DOC[];
#define drgn_TypeKind_ENUM_DOC (char *)drgn_TypeKind_ENUM_DOC
extern const char drgn_TypeKind_TYPEDEF_DOC[];
#define drgn_TypeKind_TYPEDEF_DOC (char *)drgn_TypeKind_TYPEDEF_DOC
extern const char drgn_TypeKind_POINTER_DOC[];
#define drgn_TypeKind_POINTER_DOC (char *)drgn_TypeKind_POINTER_DOC
extern const char drgn_TypeKind_ARRAY_DOC[];
#define drgn_TypeKind_ARRAY_DOC (char *)drgn_TypeKind_ARRAY_DOC
extern const char drgn_TypeKind_FUNCTION_DOC[];
#define drgn_TypeKind_FUNCTION_DOC (char *)drgn_TypeKind_FUNCTION_DOC
extern const char drgn_PrimitiveType_DOC[];
#define drgn_PrimitiveType_DOC (char *)drgn_PrimitiveType_DOC
extern const char drgn_PrimitiveType_C_VOID_DOC[];
#define drgn_PrimitiveType_C_VOID_DOC (char *)drgn_PrimitiveType_C_VOID_DOC
extern const char drgn_PrimitiveType_C_CHAR_DOC[];
#define drgn_PrimitiveType_C_CHAR_DOC (char *)drgn_PrimitiveType_C_CHAR_DOC
extern const char drgn_PrimitiveType_C_SIGNED_CHAR_DOC[];
#define drgn_PrimitiveType_C_SIGNED_CHAR_DOC (char *)drgn_PrimitiveType_C_SIGNED_CHAR_DOC
extern const char drgn_PrimitiveType_C_UNSIGNED_CHAR_DOC[];
#define drgn_PrimitiveType_C_UNSIGNED_CHAR_DOC (char *)drgn_PrimitiveType_C_UNSIGNED_CHAR_DOC
extern const char drgn_PrimitiveType_C_SHORT_DOC[];
#define drgn_PrimitiveType_C_SHORT_DOC (char *)drgn_PrimitiveType_C_SHORT_DOC
extern const char drgn_PrimitiveType_C_UNSIGNED_SHORT_DOC[];
#define drgn_PrimitiveType_C_UNSIGNED_SHORT_DOC (char *)drgn_PrimitiveType_C_UNSIGNED_SHORT_DOC
extern const char drgn_PrimitiveType_C_INT_DOC[];
#define drgn_PrimitiveType_C_INT_DOC (char *)drgn_PrimitiveType_C_INT_DOC
extern const char drgn_PrimitiveType_C_UNSIGNED_INT_DOC[];
#define drgn_PrimitiveType_C_UNSIGNED_INT_DOC (char *)drgn_PrimitiveType_C_UNSIGNED_INT_DOC
extern const char drgn_PrimitiveType_C_LONG_DOC[];
#define drgn_PrimitiveType_C_LONG_DOC (char *)drgn_PrimitiveType_C_LONG_DOC
extern const char drgn_PrimitiveType_C_UNSIGNED_LONG_DOC[];
#define drgn_PrimitiveType_C_UNSIGNED_LONG_DOC (char *)drgn_PrimitiveType_C_UNSIGNED_LONG_DOC
extern const char drgn_PrimitiveType_C_LONG_LONG_DOC[];
#define drgn_PrimitiveType_C_LONG_LONG_DOC (char *)drgn_PrimitiveType_C_LONG_LONG_DOC
extern const char drgn_PrimitiveType_C_UNSIGNED_LONG_LONG_DOC[];
#define drgn_PrimitiveType_C_UNSIGNED_LONG_LONG_DOC (char *)drgn_PrimitiveType_C_UNSIGNED_LONG_LONG_DOC
extern const char drgn_PrimitiveType_C_BOOL_DOC[];
#define drgn_PrimitiveType_C_BOOL_DOC (char *)drgn_PrimitiveType_C_BOOL_DOC
extern const char drgn_PrimitiveType_C_FLOAT_DOC[];
#define drgn_PrimitiveType_C_FLOAT_DOC (char *)drgn_PrimitiveType_C_FLOAT_DOC
extern const char drgn_PrimitiveType_C_DOUBLE_DOC[];
#define drgn_PrimitiveType_C_DOUBLE_DOC (char *)drgn_PrimitiveType_C_DOUBLE_DOC
extern const char drgn_PrimitiveType_C_LONG_DOUBLE_DOC[];
#define drgn_PrimitiveType_C_LONG_DOUBLE_DOC (char *)drgn_PrimitiveType_C_LONG_DOUBLE_DOC
extern const char drgn_PrimitiveType_C_SIZE_T_DOC[];
#define drgn_PrimitiveType_C_SIZE_T_DOC (char *)drgn_PrimitiveType_C_SIZE_T_DOC
extern const char drgn_PrimitiveType_C_PTRDIFF_T_DOC[];
#define drgn_PrimitiveType_C_PTRDIFF_T_DOC (char *)drgn_PrimitiveType_C_PTRDIFF_T_DOC
extern const char drgn_Qualifiers_DOC[];
#define drgn_Qualifiers_DOC (char *)drgn_Qualifiers_DOC
extern const char drgn_Qualifiers_NONE_DOC[];
#define drgn_Qualifiers_NONE_DOC (char *)drgn_Qualifiers_NONE_DOC
extern const char drgn_Qualifiers_CONST_DOC[];
#define drgn_Qualifiers_CONST_DOC (char *)drgn_Qualifiers_CONST_DOC
extern const char drgn_Qualifiers_VOLATILE_DOC[];
#define drgn_Qualifiers_VOLATILE_DOC (char *)drgn_Qualifiers_VOLATILE_DOC
extern const char drgn_Qualifiers_RESTRICT_DOC[];
#define drgn_Qualifiers_RESTRICT_DOC (char *)drgn_Qualifiers_RESTRICT_DOC
extern const char drgn_Qualifiers_ATOMIC_DOC[];
#define drgn_Qualifiers_ATOMIC_DOC (char *)drgn_Qualifiers_ATOMIC_DOC
extern const char drgn_sizeof_DOC[];
#define drgn_sizeof_DOC (char *)drgn_sizeof_DOC
extern const char drgn_offsetof_DOC[];
#define drgn_offsetof_DOC (char *)drgn_offsetof_DOC
extern const char drgn_FaultError_DOC[];
#define drgn_FaultError_DOC (char *)drgn_FaultError_DOC
extern const char drgn_FaultError_message_DOC[];
#define drgn_FaultError_message_DOC (char *)drgn_FaultError_message_DOC
extern const char drgn_FaultError_address_DOC[];
#define drgn_FaultError_address_DOC (char *)drgn_FaultError_address_DOC
extern const char drgn_MissingDebugInfoError_DOC[];
#define drgn_MissingDebugInfoError_DOC (char *)drgn_MissingDebugInfoError_DOC
extern const char drgn_ObjectAbsentError_DOC[];
#define drgn_ObjectAbsentError_DOC (char *)drgn_ObjectAbsentError_DOC
extern const char drgn_OutOfBoundsError_DOC[];
#define drgn_OutOfBoundsError_DOC (char *)drgn_OutOfBoundsError_DOC
extern const char drgn__linux_helper_radix_tree_lookup_DOC[];
#define drgn__linux_helper_radix_tree_lookup_DOC (char *)drgn__linux_helper_radix_tree_lookup_DOC
extern const char drgn__linux_helper_per_cpu_ptr_DOC[];
#define drgn__linux_helper_per_cpu_ptr_DOC (char *)drgn__linux_helper_per_cpu_ptr_DOC
extern const char drgn__linux_helper_idle_task_DOC[];
#define drgn__linux_helper_idle_task_DOC (char *)drgn__linux_helper_idle_task_DOC
extern const char drgn__linux_helper_idr_find_DOC[];
#define drgn__linux_helper_idr_find_DOC (char *)drgn__linux_helper_idr_find_DOC
extern const char drgn__linux_helper_find_pid_DOC[];
#define drgn__linux_helper_find_pid_DOC (char *)drgn__linux_helper_find_pid_DOC
extern const char drgn__linux_helper_pid_task_DOC[];
#define drgn__linux_helper_pid_task_DOC (char *)drgn__linux_helper_pid_task_DOC
extern const char drgn__linux_helper_find_task_DOC[];
#define drgn__linux_helper_find_task_DOC (char *)drgn__linux_helper_find_task_DOC
extern const char drgn__linux_helper_kaslr_offset_DOC[];
#define drgn__linux_helper_kaslr_offset_DOC (char *)drgn__linux_helper_kaslr_offset_DOC
extern const char drgn__linux_helper_pgtable_l5_enabled_DOC[];
#define drgn__linux_helper_pgtable_l5_enabled_DOC (char *)drgn__linux_helper_pgtable_l5_enabled_DOC
